from .canvas import Canvas
import math
import random
import time

BLACK=1
WHITE=2

board = [
        [0,0,0,0,0,0],
        [0,0,0,0,0,0],
        [0,0,1,2,0,0],
        [0,0,2,1,0,0],
        [0,0,0,0,0,0],
        [0,0,0,0,0,0],
]

def can_place_x_y(board, stone, x, y):
    """
    石を置けるかどうかを調べる関数。
    board: 2次元配列のオセロボード
    x, y: 石を置きたい座標 (0-indexed)
    stone: 現在のプレイヤーの石 (1: 黒, 2: 白)
    return: 置けるなら True, 置けないなら False
    """
    if board[y][x] != 0:
        return False  # 既に石がある場合は置けない

    opponent = 3 - stone  # 相手の石 (1なら2、2なら1)
    directions = [(-1, -1), (-1, 0), (-1, 1), (0, -1), (0, 1), (1, -1), (1, 0), (1, 1)]

    for dx, dy in directions:
        nx, ny = x + dx, y + dy
        found_opponent = False

        while 0 <= nx < len(board[0]) and 0 <= ny < len(board) and board[ny][nx] == opponent:
            nx += dx
            ny += dy
            found_opponent = True

        if found_opponent and 0 <= nx < len(board[0]) and 0 <= ny < len(board) and board[ny][nx] == stone:
            return True  # 石を置ける条件を満たす

    return False

def can_place(board, stone):
    """
    石を置ける場所を調べる関数。
    board: 2次元配列のオセロボード
    stone: 現在のプレイヤーの石 (1: 黒, 2: 白)
    """
    for y in range(len(board)):
        for x in range(len(board[0])):
            if can_place_x_y(board, stone, x, y):
                return True
    return False

def random_place(board, stone):
    """
    石をランダムに置く関数。
    board: 2次元配列のオセロボード
    stone: 現在のプレイヤーの石 (1: 黒, 2: 白)
    """
    while True:
        x = random.randint(0, len(board[0]) - 1)
        y = random.randint(0, len(board) - 1)
        if can_place_x_y(board, stone, x, y):
            return x, y

def copy(board):
    """
    盤面をコピーする関数。
    board: 2次元配列のオセロボード
    """
    return [row[:] for row in board]


def move_stone(board, stone, x, y):
    """
    石を置き、ひっくり返す関数。
    board: 2次元配列のオセロボード
    x, y: 石を置きたい座標 (0-indexed)
    stone: 現在のプレイヤーの石 (1: 黒, 2: 白)
    return:
    """
    moves = [copy(board)]*3
    if not can_place_x_y(board, stone, x, y):
        return moves  # 置けない場合は何もしない

    board[y][x] = stone  # 石を置く
    moves.append(copy(board))
    opponent = 3 - stone
    directions = [(-1, -1), (-1, 0), (-1, 1), (0, -1), (0, 1), (1, -1), (1, 0), (1, 1)]
    flipped_count = 0

    for dx, dy in directions:
        nx, ny = x + dx, y + dy
        stones_to_flip = []

        while 0 <= nx < len(board[0]) and 0 <= ny < len(board) and board[ny][nx] == opponent:
            stones_to_flip.append((nx, ny))
            nx += dx
            ny += dy

        if stones_to_flip and 0 <= nx < len(board[0]) and 0 <= ny < len(board) and board[ny][nx] == stone:
            for flip_x, flip_y in stones_to_flip:
                board[flip_y][flip_x] = stone
                moves.append(copy(board))
                flipped_count += 1

    return moves


class PandaAI(object):

    def face(self):
        return "🐼"

    def place(self, board, stone):
        x, y = random_place(board, stone)
        return x, y


def draw_board(canvas, board):
    ctx = canvas.getContext("2d")
    grid = width // len(board)
    for y, line in enumerate(board):
        for x, stone in enumerate(line):
            cx = x * grid + grid // 2
            cy = y * grid + grid // 2
            if stone != 0:
                ctx.beginPath()
                ctx.arc(cx, cy, grid//2, 0, 2 * math.pi) # 円の描画
                ctx.fillStyle = "black" if stone == 1 else "white"
                ctx.fill()

width=300

def draw_board_moves(canvas, moves):
    for board in moves:
        draw_board(canvas, board)

def play_othello(ai=None, board=None):
    if board is None:
        board = [
            [0,0,0,0,0,0],
            [0,0,0,0,0,0],
            [0,0,1,2,0,0],
            [0,0,2,1,0,0],
            [0,0,0,0,0,0],
            [0,0,0,0,0,0],
        ]
    if ai is None:
        ai = PandaAI()

    def redraw(canvas, x, y):
        nonlocal board, ai
        N = len(board)
        grid = width // N
        x = x // grid
        y = y // grid
        if not can_place_x_y(board, BLACK, x, y):
            print('そこに置けません', (x, y))
            return

        moves = []
        moves.extend(move_stone(board, BLACK, x, y))

        if can_place(board, WHITE):
            x, y = ai.place(board, WHITE)
            if not can_place_x_y(board, WHITE, x, y):
                print(f'{ai.face()}は、置けないところに置こうとしました', (x, y))
                print('反則負けです')
                return
            print(f'{ai.face()}は', (x, y), 'におきました。')
            moves.extend(move_stone(board, WHITE, x, y))
        else:
            print(f'{ai.face()}はどこにも置けないのでスキップします')

        while not can_place(board, BLACK):
            if can_place(board, WHITE):
                print(f'あなたはどこにも置けないのでスキップします')
                x, y = ai.place(board, WHITE)
                print(f'{ai.face()}は', (x, y), 'におきました。')
                moves.extend(move_stone(board, WHITE, x, y))
            else:
                black = sum(row.count(BLACK) for row in board)
                white = sum(row.count(WHITE) for row in board)
                print(f'黒: {black}, 白: {white}', end=' ')
                if black > white:
                    print('黒の勝ち')
                elif black < white:
                    print('白の勝ち')
                else:
                    print('引き分け')
                break
        draw_board_moves(canvas, moves)


    canvas = Canvas(background='green', grid=width//6, width=width, height=width, onclick=redraw)
    draw_board(canvas, board)

    display(canvas)


def count_stone(board):
    black = sum(row.count(BLACK) for row in board)
    white = sum(row.count(WHITE) for row in board)
    return black, white

def run_othello(blackai=None, whiteai=None, board=None):
    if board is None:
        board = [
            [0,0,0,0,0,0],
            [0,0,0,0,0,0],
            [0,0,1,2,0,0],
            [0,0,2,1,0,0],
            [0,0,0,0,0,0],
            [0,0,0,0,0,0],
        ]
    if blackai is None:
        blackai = PandaAI()

    if whiteai is None:
        whiteai = PandaAI()
        print(f'{whiteai.face()}が相手するよ！覚悟しな！')

    black_time = 0
    white_time = 0
    moved = True
    while moved and can_place(board, BLACK) and can_place(board, WHITE):
        moved = False
        if can_place(board, BLACK):
            start = time.time()
            x, y = blackai.place(copy(board), BLACK)
            black_time += time.time() - start
            if not can_place_x_y(board, BLACK, x, y):
                print(f'{blackai.face()}は、置けないところに置こうとしました', (x, y))
                print('反則負けです')
                return
            move_stone(board, BLACK, x, y)
            black, white = count_stone(board)
            print(f'{blackai.face()}は{(x, y)}におきました。黒: {black}, 白: {white}')
            moved = True
        else:
            print(f'{blackai.face()}は、どこにも置けないのでスキップします')

        if can_place(board, WHITE):
            start = time.time()
            x, y = whiteai.place(copy(board), WHITE)
            white_time += time.time() - start
            if not can_place_x_y(board, WHITE, x, y):
                print(f'{whiteai.face()}は、置けないところに置こうとしました', (x, y))
                print('反則負けです')
                return
            move_stone(board, WHITE, x, y)
            black, white = count_stone(board)
            print(f'{whiteai.face()}は{(x, y)}におきました。黒: {black}, 白: {white}')
            moved = True
        else:
            print(f'{whiteai.face()}は、どこにも置けないのでスキップします')
        
    black, white = count_stone(board)
    print(f'最終結果: 黒: {black}, 白: {white}', end=' ')
    if black > white:
        print(f'黒{blackai.face()}の勝ち')
    elif black < white:
        print(f'白{whiteai.face()}の勝ち')
    else:
        print('引き分け')
    print(f'思考時間: 黒: {black_time:.5f}秒, 白: {white_time:.5f}秒')






import builtins
import numpy as np
import json
import os
import sys
import shlex
from base64 import b64encode
from binascii import a2b_base64
import traceback
import IPython
from IPython.display import HTML, JSON, display

from .google_check import google_output

INSTALLED = False


def _install():
    global INSTALLED
    if not INSTALLED and google_output:
        os.system('apt-get -y install imagemagick')
        os.system('apt-get -y install pngquant')
        INSTALLED = True


def _convert(file, width=None, height=None):
    if height is None:
        height = width * 10 if width else None
    if width is None:
        width = height * 10 if height else None
    if width and height:
        file2 = file.replace('.', '_.')
        os.system(f'convert -resize {width}x{height} {file} {file2}')
        os.remove(file)
        return file2
    return file


def toDataURL(file, mimetype):
    with open(file, 'rb') as f:
        bin = f.read()
    return f'data:{mimetype};base64,'+b64encode(bin).decode()


IMAGE_CACHE = {}


def wget_dataurl(url, width=None, height=None):
    _install()
    key = f'{url}{width}x{height}'
    if key in IMAGE_CACHE:
        return IMAGE_CACHE[key]
    qurl = shlex.quote(url)
    data_url = None
    if url.endswith('.png') or url.endswith('.PNG'):
        os.system(f'wget {qurl} -O im.png')
        if not os.path.exists('im.png'):
            return False
        file = _convert('im.png', width, height)
        file2 = file.replace('.png', '-fs8.png')
        os.system(f'pngquant {file}')
        os.remove(file)
        data_url = toDataURL(file2, 'image/png')
        os.remove(file2)
    if url.endswith('.jpg') or url.endswith('.jpeg'):
        os.system(f'wget {qurl} -O im.jpg')
        if not os.path.exists('im.jpg'):
            return False
        file = _convert('im.jpg', width, height)
        data_url = toDataURL(file, 'image/jpeg')
        os.remove(file)
    if data_url is None:
        return False
    IMAGE_CACHE[key] = data_url
    return data_url


class MP4(object):
    def __init__(self, filename, width=400):
        self.filename = filename
        self.width = width

    def _repr_html_(self):
        with open(self.filename, 'rb') as fd:
            bin = fd.read()
            data_url = "data:video/mp4;base64," + b64encode(bin).decode()
            return f'''
            <video width="{self.width}" controls>
            <source src="{data_url}" type="video/mp4">
            </video>
            '''


def new_context(contexts=[]):
    class KParam(object):
        def __init__(self, name, value):
            nonlocal contexts
            self.name = name
            self.value = value
            contexts.append(self)

        def to_json(self):
            return (0, self.name, self.value)

    class KMethod(object):
        def __init__(self, name):
            self.name = name
            self.args = ()
            nonlocal contexts
            contexts.append(self)

        def __call__(self, *args):
            self.args = args

        def to_json(self):
            return (1, self.name, self.args)

    class Context(object):
        def __setattr__(self, name, value):
            KParam(name, value)
            return None

        def __getattr__(self, name):
            return KMethod(name)

    return Context()

# HTML


def html_img(key, data_url):
    return f'<img id="{key}" src="{data_url}">'


ANIME = '''
<div style="display:none;">
IMG
</div>
<canvas id="canvas" width="400" height="300" style="background-color:rgb(0,0,0)">
</canvas>
'''

MOVIE = '''
<progress id="prog_bar" value="0" max="100"></progress>
<div style="display:none;">
IMG
<canvas id="canvas" width="400" height="300" style="background-color:rgb(0,0,0)">
</canvas>
</div>
'''


def make_html(canvas, base=ANIME):
    images = ''
    if len(canvas.images) > 0:
        ss = [html_img(key, data_url)
              for key, data_url in canvas.images.items()]
        images = '\n'.join(ss)
    newid = f'"canvas{id(canvas)}"'
    base = base.replace('"canvas"', newid)
    base = base.replace('400', f'{canvas.width}')
    base = base.replace('300', f'{canvas.height}')
    base = base.replace('IMG', images)
    return base


DRAW_JS = '''
const canvas = document.getElementById("canvas");
const dpr = window.devicePixelRatio || 1;
const width = canvas.width;
const height = canvas.height;
canvas.width *= dpr;
canvas.height *= dpr;
canvas.style.width = width + 'px';
canvas.style.height = height + 'px';

// Canvasの描画自体を拡大
const ctx = canvas.getContext('2d');
ctx.scale(dpr, dpr);

const draw = (data) => {
    if(data.length===0) {
        return;
    }
    for(const op of data) {
        if(op[0] === 0) {
            ctx[op[1]] = op[2];
        }
        else{
            if(op[1] === 'drawImage' && typeof op[2][0] === 'string') {
                console.log(op[2][0]);
                op[2][0] = document.getElementById(op[2][0]);
            }
            ctx[op[1]](...op[2]);
        }
    }
};

var frame = [[]];
'''

ANIME_JS = '''
var tm = undefined;
const start_anime = (repeat) => {
    if(tm !== undefined) {
        clearInterval(tm);
    }
    var frame_idx = 0;
    tm = setInterval(() => {
        draw(frame[frame_idx]);
        frame_idx += 1;
        if(frame_idx >= frame.length) {
            if(repeat > 0) {
                frame_idx = 0;
                repeat -= 1;
            }
            else {
                clearInterval(tm);
            }
        }
    }, 100);
};
start_anime(10);
'''

CLICK_JS = '''
var mouse = {x: 0, y: 0};
canvas.addEventListener('mousemove', function(e) {
  mouse.x = e.pageX - this.offsetLeft
  mouse.y = e.pageY - this.offsetTop
});

const redraw = (x, y) => {
    (async function() {
        const result = await google.colab.kernel.invokeFunction(
            'notebook.click',
            [x, y],
            {}); // kwargs
        const updated = result.data['application/json'].result;
        if(updated.length > 0) {
            frame = updated;
            start_anime(0);
        }
    })();
};

canvas.onmousedown = () => {
  redraw(mouse.x, mouse.y);
}
'''

MOVIE_JS = '''
var frame_count=0;
const bar = document.getElementById('prog_bar');
const save = () => {
    draw(frame[0]);
    const idx = frame_count++;
    const dataURL = canvas.toDataURL();
    (async function() {
        const result = await google.colab.kernel.invokeFunction(
            'notebook.save',
            [idx, dataURL],
            {}); // kwargs
        const jsondata = result.data['application/json']
        const updated = jsondata.result;
        bar.value = jsondata.value;
        bar.max = jsondata.max;    
        if(updated.length > 0) {
            frame = updated;
            save();
        }
    })();
};
save()
'''


def make_js(canvas, asm, fps=0, onclick=None):
    js = DRAW_JS.replace('[[]]', json.dumps(asm))
    newid = f'"canvas{id(canvas)}"'
    js = js.replace('"canvas"', newid)
    if fps > 0:
        js += ANIME_JS.replace('100', f'{1000//fps}')
    else:
        js += MOVIE_JS
    if onclick is not None:
        js += CLICK_JS
    return f'<script>\n{js}\n</script>'


def safe(f):
    def safe_fn(*args):
        nonlocal f
        try:
            return f(*args)
        except:
            traceback.print_exc()
            return JSON({
                'result': [[]]
            })
    return safe_fn


class Canvas(object):
    def __init__(self, width=400, height=300, background='white', fps=5, onclick=None):
        self.width = width
        self.height = height
        self.images = {}
        self.buffers = []
        self.background = background
        self.fps = fps
        self.onclick_fn = onclick
        self.filename = 'canvas.mp4'
        self.displayed = False
        if google_output:
            google_output.register_callback(
                'notebook.click', safe(self.click))
            google_output.register_callback(
                'notebook.save', safe(self.save))

    def loadImage(self, image_key, url, width=None, height=None):
        self.images[image_key] = wget_dataurl(url, width=width, height=height)
        return HTML(html_img(image_key, self.images[image_key]))

    def getContext(self, target='2d'):
        if self.displayed:
            self.displayed = False
            self.buffers = []
        cb = []
        self.buffers.append(cb)
        ctx = new_context(cb)
        ctx.clearRect(0, 0, self.width, self.height)
        ctx.fillStyle = self.background
        ctx.fillRect(0, 0, self.width, self.height)
        return ctx

    def asm(self):
        if len(self.buffers) == 0:
            return [[]]
        return [[c.to_json() for c in cb] for cb in self.buffers]

    def _repr_html_(self):
        self.displayed = True
        return make_html(self) + make_js(self, self.asm(), self.fps, self.onclick_fn)

    def click(self, x, y):
        self.buffers = []
        self.onclick_fn(self, x, y)
        return JSON({
            'result': self.asm()
        })

    def save_movie(self, filename=None, fps=None):
        if filename is not None:
            self.filename = filename
        if fps is not None:
            self.fps = int(fps)
        i = 0
        while True:
            fname = f'frame{i:04d}.png'
            if not os.path.exists(fname):
                break
            os.remove(fname)
            i += 1
        first_frame = [[c.to_json() for c in self.buffers[0]]]
        display(HTML(make_html(self, MOVIE) + make_js(self, first_frame)))

    def save(self, idx, dataURI):
        _, _, dataURI = dataURI.partition("base64,")
        binary_data = a2b_base64(dataURI)
        fname = f'frame{idx:04d}.png'
        # print(fname)
        with open(fname, 'wb') as fd:
            fd.write(binary_data)
        update = []
        msg = f'Saved'
        if idx < len(self.buffers):
            cb = self.buffers[idx]
            update = [[c.to_json() for c in cb]]
            value = idx
            msg = f'Saved [{idx}/{len(self.buffers)}] {fname}'
        else:
            self._save_movie()
            value = len(self.buffers)
            msg = f'Saved {self.filename}'
        return JSON({
            'result': update,
            'max': len(self.buffers),
            'value': value,
            'meg': msg,
        })

    def _save_movie(self):
        filename = shlex.quote(self.filename)
        framerate = int(self.fps)
        if os.path.exists(filename):
            os.remove(filename)
        os.system(
            f'ffmpeg -y -framerate {framerate} -i frame%04d.png -vcodec libx264 -pix_fmt yuv420p {filename}')
        if os.path.exists(filename):
            print(f'Saved {filename}')
            display(MP4(filename, self.width))


a = np.array([[2, 1, 3], [3, 2, 1]])


def draw_np1d(ctx, a, x=0, y=0, width=400, height=300, ensure_square=True, margin=1, rgb=(255, 219, 237), min=None, max=None):
    max = max or a.max()
    min = min or a.min()
    w = len(a)
    a = a - min
    dx = width//w
    a = a * height / (max-min)
    ca = np.array(rgb)
    ca = ca / (max-min)
    for i, dy in enumerate(a):
        c = (((ca * dy) % 128) + 128).astype(int)
        ctx.fillStyle = f'rgb({c[0]},{c[1]},{c[2]})'
        ctx.fillRect(x+i*dx, y+height-dy, dx-margin, dy)


def draw_np2d(ctx, a, x=0, y=0, width=400, height=300, ensure_square=True, margin=1, rgb=(0, 128, 0), min=None, max=None):
    h, w = a.shape
    min = min or a.min()
    max = max or a.max()
    a = a - min
    dx = width//w
    dy = height//h
    if ensure_square:
        dx = builtins.min(dx, dy)
        dy = builtins.min(dx, dy)
    ca = np.array(rgb)
    ca = ca / (max-min)
    for wi in range(w):
        for hi in range(h):
            c = (ca*a[hi][wi]).astype(int)
            ctx.fillStyle = f'rgb({c[0]},{c[1]},{c[2]})'
            ctx.fillRect(x+wi*dx, y+hi*dy, dx-margin, dy-margin)


def draw_np(ctx, a, x=0, y=0, width=400, height=300, ensure_square=True, margin=1, rgb=(255, 219, 237), min=None, max=None):
    if not isinstance(a, np.ndarray):
        a = np.array(list(a))
    if len(a.shape) == 2:
        draw_np2d(ctx, a, x, y, width, height,
                  ensure_square, margin, rgb, min, max)
    else:
        draw_np1d(ctx, a, x, y, width, height,
                  ensure_square, margin, rgb, min, max)



import subprocess
import ffmpeg
import sys

_installed = False

def is_ffmpeg_installed():
    global _installed
    if _installed == True:
        return True
    try:
        # FFmpegがインストールされているかを確認
        result = subprocess.run(["ffmpeg", "-version"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        _installed = True
        return True
    except subprocess.CalledProcessError:
        return False
    except FileNotFoundError:
        return False

def install_ffmpeg(sync=False):
    if is_ffmpeg_installed():
        return

    # # パッケージリストを更新
    # subprocess.Popen(["apt-get", "update"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    # FFmpegをインストール
    if sync:
        subprocess.run(["apt-get", "install", "-y", "ffmpeg"], check=True)
        print("FFmpeg installation complete.")    
    else:
        subprocess.Popen(["apt-get", "install", "-y", "ffmpeg"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print("FFmpeg installation started in the background.")

def create_video(uuid0, framerate, filename):
    install_ffmpeg(sync=True)
    print(f'!ffmpeg -y -framerate {framerate} -i canvas{uuid0}/frame%04d.png -vcodec libx264 -pix_fmt yuv420p {filename}')
    # FFmpegコマンドを実行し、stderrをキャプチャ
    process = (
        ffmpeg
        .input(f'canvas{uuid0}/frame%04d.png', framerate=framerate)
        .output(filename, vcodec='libx264', pix_fmt='yuv420p')
        .run_async(pipe_stdout=True, pipe_stderr=True, overwrite_output=True)
    )

    # ストリームを読み取ってstderrを表示
    stdout, stderr = process.communicate()
    # print('@', stdout, stderr)

    # 標準エラー出力を表示
    if stderr:
        filename = filename.replace('.', '_')
        with open(f'{filename}_log.txt', 'w') as f:
            f.write(stderr.decode('utf-8'))



import requests
import os
from urllib.parse import urlparse
from PIL import Image
from io import BytesIO
from base64 import b64encode

uuid0 = -1

def download_image(url, save_dir='images'):
    global uuid0
    # 画像をダウンロード
    response = requests.get(url)
    response.raise_for_status()  # エラーチェック

    # URLからファイル名を抽出
    parsed_url = urlparse(url)
    filename = os.path.basename(parsed_url.path)
    
    # ファイル名が存在しない場合は、画像の種類から生成
    if not filename:
        img = Image.open(BytesIO(response.content))
        ext = img.format.lower()
        uuid0 += 1
        filename = f'image{uuid0}.{ext}'
    
    # 保存ディレクトリを作成
    if not os.path.exists(save_dir):
        os.makedirs(save_dir)

    # ファイルを保存
    save_path = os.path.join(save_dir, filename)
    with open(save_path, 'wb') as file:
        file.write(response.content)
    print(f'Image saved to {save_path}')
    return save_path

def resize_image(image_path, new_width=None, new_height=None):
    # 画像を開く
    if new_width is None and new_height is None:
        return image_path

    with Image.open(image_path) as img:
        # 現在の幅と高さを取得
        width, height = img.size
        print(f"Original width: {width}, Original height: {height}")
        if new_height is None:
            new_height = int(height * new_width / width)
        if new_width is None:
            new_width = int(width * new_height / height)

        # 画像をリサイズ
        resized_img = img.resize((new_width, new_height))

        path, _, ext = image_path.rpartition('.')
        output_path = f'{path}_{new_width}x{new_height}.{ext}'
        # リサイズされた画像を保存
        resized_img.save(output_path)
        print(f"Resized image saved as: {output_path}")
        return output_path

_CACHE = {}

def load_image(url_or_imagepath, width=None, height=None):
    global _CACHE
    if url_or_imagepath.startswith('http://') or url_or_imagepath.startswith('https://'):
        if url_or_imagepath not in _CACHE:
            _CACHE[url_or_imagepath] = download_image(url_or_imagepath)
        save_path = _CACHE[url_or_imagepath]
    else:
        save_path = url_or_imagepath
    if width is None and height is None:
        return save_path
    key = (save_path, width, height)
    if key not in _CACHE:
        _CACHE[key] = resize_image(save_path, width, height)
    return _CACHE[key]

mime_types = {
    'jpg': 'image/jpeg',
    'jpeg': 'image/jpeg',
    'png': 'image/png',
    'gif': 'image/gif',
    'bmp': 'image/bmp',
    'tiff': 'image/tiff',
    'webp': 'image/webp',
    'svg': 'image/svg+xml'
}

def file_to_mime(path):
    path, _, ext = path.rpartition('.')

    # 拡張子の前後の空白を削除し、小文字に変換
    ext = ext.strip().lower()
    
    # MIMEタイプを取得（存在しない場合は'image/png'）
    return mime_types.get(ext, 'image/png')
 

def data_url(file, width=None, height=None):
    file = load_image(file, width, height)
    with open(file, 'rb') as f:
        bin = f.read()
    mimetype = file_to_mime(file)
    return f'data:{mimetype};base64,'+b64encode(bin).decode()




from setuptools import setup, find_packages

'''
python3 -m unittest
vim setup.py
rm -rf dist/
python3 setup.py sdist bdist_wheel
twine upload --repository pypi dist/*
'''

def _requires_from_file(filename):
    return open(filename).read().splitlines()


setup(name="kogi-canvas",
      version="0.2.1",
      license='Apache',
      author='Kimio Kuramitsu',
      description="Canvas from KOGI",
      url="https://github.com/kuramitsulab/kogi-canvas",
      packages=['kogi_canvas'],
      # package_dir={"": "src"},
      package_data={'kogi_canvas': ['./*.*']},
      install_requires=_requires_from_file('requirements.txt'),
      classifiers=[
          'Development Status :: 4 - Beta',
          'License :: OSI Approved :: Apache Software License',
          'Framework :: IPython',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Programming Language :: Python :: 3.11',
          'Intended Audience :: Education',
      ],
      )
